import exit from 'exit';
import queue from 'queue';
import ProgressBar from 'progress';
import {gray, error, n, checkbox} from '../../utils/decorators';
import Config from '../../utils/config';

const config = new Config('./blinkloader.json');

const fs = require('fs');
const ncp = require('copy-paste');
const AWS = require('aws-sdk');
const glob = require('glob');

const q = queue({concurrency: 1});

export default clientConfig => {
	let {AWS_REGION, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY} = process.env;

	if (!AWS_REGION || !AWS_ACCESS_KEY_ID || !AWS_SECRET_ACCESS_KEY) {
		error('blinkloader relies on your Amazon credentials, some of them were not provided');
		error(gray('AWS_REGION, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY are required'));
		n();
		exit();
	}

	const gzipExts = ['js', 'css', 'html', 'htm', 'png', 'jpg', 'jpeg', 'gif'];

	const s3 = new AWS.S3();

	q.push(cb => {
		s3.createBucket({Bucket: clientConfig.domain}, err => {
			if (err && err.code === 'BucketAlreadyOwnedByYou') {
				checkbox(`bucket ${gray(clientConfig.domain)} already exists`);
				AWS_REGION = err.region;
				cb();
			} else if (err && err.code === 'UnknownEndpoint') {
				error('blinkloader relies on your Amazon credentials, some of them are not valid');
				n();
				exit();
			} else if (err) {
				error('something went wrong');
				console.log(JSON.stringify(err));
				n();
				exit();
			} else {
				checkbox(`created new s3 bucket ${gray(clientConfig.domain)}`);
				cb();
			}
		});
	});

	q.push(cb => {
		glob(`${config.getSiteOptimizedDir()}/**/*.*`, {}, (er, files) => {
			const filesMap = files.map(file => {
				let ContentType;
				let CacheControl;
				let ContentEncoding;
				const fileExtension = file.split('.')[file.split('.').length - 1];

				if (config.getOptimizations().includes('gzip') && gzipExts.includes(fileExtension)) {
					ContentEncoding = 'gzip';
				}

				if (fileExtension === 'png') {
					ContentType = 'image/png';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'jpg' || fileExtension === 'jpeg') {
					ContentType = 'image/jpeg';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'webp') {
					ContentType = 'image/webp';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'gif') {
					ContentType = 'image/gif';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'svg') {
					ContentType = 'image/svg+xml';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'css') {
					ContentType = 'text/css';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'json') {
					ContentType = 'application/json';
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'html' || fileExtension === 'htm') {
					ContentType = 'text/html';
					CacheControl = 'no-cache';
				} else if (fileExtension === 'js') {
					CacheControl = 'max-age=31536000';
				} else if (fileExtension === 'webm') {
					CacheControl = 'max-age=31536000';
				}

				return {file, ContentType, CacheControl, ContentEncoding, target: file.replace(`${config.getSiteOptimizedDir()}/`, '')};
			});

			let totalUploaded = 0;
			const totalFiles = files.length;
			checkbox(`started syncing the bucket with ${gray(config.getSiteOptimizedDir())}`);
			const bar = new ProgressBar('              uploading [:bar] :percent', {
				total: totalFiles,
				complete: '=',
				incomplete: ' ',
				width: 20
			});

			filesMap.forEach(fileMap => {
				fs.readFile(fileMap.file, (err, fileContent) => {
					const params = {
						Body: fileContent,
						CacheControl: fileMap.CacheControl,
						ContentEncoding: fileMap.ContentEncoding,
						ContentType: fileMap.ContentType,
						Bucket: clientConfig.domain,
						Key: fileMap.target
					};
					s3.upload(params, uploaderr => {
						if (uploaderr) {
							error('something went wrong, couldn\'t send assets to the bucket');
							console.log(JSON.stringify(uploaderr));
							n();
							exit();
						}
						totalUploaded++;
						bar.tick();
						if (bar.complete) {
							n();
						}

						if (totalUploaded === totalFiles) {
							cb();
						}
					});
				});
			});
		});
	});

	let errorPageKey = null;
	q.push(cb => {
		fs.stat(`${config.getSiteOptimizedDir()}/404/index.html`, (err, exists) => {
			if (exists) {
				errorPageKey = '404/index.html';
				cb();
			} else {
				fs.stat(`${config.getSiteOptimizedDir()}/404.html`, (err, exists) => {
					if (exists) {
						errorPageKey = '404.html';
					}
					cb();
				});
			}
		});
	});

	q.push(cb => {
		const staticHostParams = {
			Bucket: clientConfig.domain,
			WebsiteConfiguration: {
				IndexDocument: {
					Suffix: 'index.html'
				}
			}
		};

		if (errorPageKey) {
			staticHostParams.WebsiteConfiguration.ErrorDocument = {Key: errorPageKey};
		}

		s3.putBucketWebsite(staticHostParams, err => {
			if (err) {
				error('something went wrong, couldn\'t set static website parameters');
				console.log(JSON.stringify(err));
				exit();
				n();
			} else {
				checkbox('set up static website parameters');
			}
			cb();
		});
	});

	q.push(cb => {
		const params = {
			Bucket: clientConfig.domain,
			Policy: '{ "Version":"2012-10-17", "Statement":[{ "Sid":"PublicReadGetObject", "Effect":"Allow", "Principal": "*", "Action":["s3:GetObject"], "Resource":["arn:aws:s3:::' + clientConfig.domain + '/*"] }] }'
		};
		s3.putBucketPolicy(params, err => {
			if (err) {
				error('something went wrong, couldn\'t set the bucket policy');
				exit();
				n();
			} else {
				checkbox('set up the bucket policy to make your website available');
			}
			cb();
		});
	});

	q.start(() => {
		n();
		const websiteLink = `http://${clientConfig.domain}.s3-website-${AWS_REGION}.amazonaws.com`;
		ncp.copy(websiteLink, () => {
			checkbox(`Woohoo, it's deployed!`);
			checkbox(`${gray(websiteLink)}`);
			checkbox(`copied the link to clipboard, use Ctrl-V to paste anywhere`);
			n();
		});
	});
};
