import exit from 'exit';
import inquirer from 'inquirer';
import jsonfile from 'jsonfile';
import cardinal from 'cardinal';

import {p, n, green} from '../utils/decorators';
import {validateUrl} from '../utils/utils';

const pathExists = require('path-exists');

const staticWebsiteIntegrations = ['static-website-via-s3', 'self-hosted-static-website'];

const envApiKey = process.env.BLINKLOADER_API_KEY || '';

module.exports = () => {
	const isBlinkDir = Boolean(pathExists.sync('./blinkloader.json'));
	const questions = [{
		type: 'confirm',
		name: 'init',
		message: 'Configuration file (./blinkloader.json) ' + (isBlinkDir ? '' : 'wasn\'t ') + 'found. Would you like to ' + (isBlinkDir ? 'override it' : 'create a new one') + '?'
	}, {
		type: 'list',
		name: 'stack',
		message: 'What\'s your stack?',
		choices: ['react', 'static-website-via-s3', 'self-hosted-static-website'],
		when: answers => answers.init ? true : exit()
	}, {
		type: 'input',
		name: 'resourceWhitelist',
		message: 'Enter resource URIs to be whitelisted (ex. "example.com, image.cdn.com/4h82f, another.cdn.com")',
		validate: values => {
			const vals = values.split(', ');
			for (let i = 0; i < vals.length; i++) {
				if (!validateUrl(vals[i])) {
					return 'Bad url: ' + vals[i];
				}
			}
			return true;
		},
		when: answers => answers.stack === 'react'
	}, {
		type: 'input',
		name: 'directory',
		message: 'What\'s the name of the static website directory (ex. dist, static, build, etc.)?',
		validate: directory => {
			if (pathExists.sync(directory)) {
				return true;
			}
			return 'Please, enter an existing directory';
		},
		when: answers => staticWebsiteIntegrations.indexOf(answers.stack) !== -1
	}, {
		type: 'checkbox',
		name: 'optimizations',
		message: 'Select desired optimizations?',
		choices: ['instant-css', 'adaptive-images', 'images', 'webp', 'async-js', 'minify', 'gzip'],
		when: answers => staticWebsiteIntegrations.indexOf(answers.stack) !== -1
	}, {
		type: 'input',
		name: 'website',
		message: 'What\'s the url of your website?',
		validate: url => {
			if (validateUrl(url)) {
				return true;
			}
			return 'Please, enter a valid url';
		}
	}, {
		type: 'confirm',
		name: 'hasApiKey',
		message: 'Do you already have the API key?'
	}, {
		type: 'input',
		name: 'apiKey',
		message: 'Please, enter your API key' + (envApiKey ? ' (press enter to use ' + green(envApiKey) + ')' : '') + ':',
		when: answers => answers.hasApiKey,
		validate: apiKey => {
			if ((apiKey.length) === 0) {
				if (envApiKey.length > 0) {
					return true;
				}
				return 'Please, enter a valid API key';
			}
			return true;
		}
	}, {
		type: 'input',
		name: 'projectId',
		message: 'Please, enter your project-id:',
		validate: pid => {
			if (pid && pid.length !== 0) {
				return true;
			}
			return 'Please, enter a valid project-id';
		}
	}];

	inquirer.prompt(questions).then(answers => {
		if (answers.apiKey === '' && envApiKey.length > 0) {
			answers.apiKey = envApiKey;
		}
		const configObject = {
			apiKey: answers.apiKey || 'YOUR_API_KEY',
			projectId: answers.projectId || 'YOUR_PROJECT_ID',
			domains: [
				answers.website.replace(/http:\/\//gi, '').replace(/https:\/\//gi, '')
			],
			stack: answers.stack
		};

		if (staticWebsiteIntegrations.indexOf(answers.stack) !== -1) {
			configObject.optimizations = answers.optimizations;
			configObject.directory = answers.directory;
		}

		if (answers.resourceWhitelist) {
			configObject.resourceWhitelist = answers.resourceWhitelist.split(', ');
		}

		jsonfile.writeFile('./blinkloader.json', configObject, {spaces: 2}, () => {
			n();
			p('Successfully created ./blinkloader.json with the following configuration.');
			p('You can always change it if needed.');
			p(cardinal.highlight(JSON.stringify(configObject, null, '  ')));
			n();
			p('Please, read our docs about how to use Blinkloader with your stack:');
			p(green('https://blinkloader.com/docs/'));
			n();
			if (staticWebsiteIntegrations.indexOf(answers.stack) !== -1) {
				p('To run the optimization locally, use this command:');
				p(green('$ blinkloader optimize'));
				n();
			}
			exit(0);
		});
	});
};
