import fs from 'fs';
import kill from 'death';
import fetch from 'node-fetch';

import decorators from '../../utils/decorators';
import Config from '../../utils/config';

import checkAPIQuota from './check-api-quota';
import createOptimization from './create-optimization';
import prepareOptimization from './prepare-optimization';
import startOptimization from './start-optimization';
import waitOptimization from './wait-optimization';
import fetchPageDownloadLinks from './fetch-page-links';
import downloadPages from './download-pages';
import deleteOptimization from './delete-optimization';
import minifyAssets from './minify-assets';
import gzipAssets from './gzip-assets';

const del = require('del');

const config = {
	pages: [],
	pageLinks: [],

	assets: [],
	assetLinks: {},

	projectID: '',
	optimizationCreated: false,
	config: new Config('./blinkloader.json'),

	fetch,
	decorators
};

const removeDir = (dir, config) => () => new Promise(resolve => {
	config.config.removeDir(dir, () => resolve(config));
});

const removeGlob = glob => config => new Promise(resolve => {
	del([glob]).then(() => {
		resolve(config);
	});
});

// Show woohoo success message
const optimizationSuccess = config => {
	config.decorators.n();
	config.decorators.log('Woohoo! Optimization complete!');
	config.decorators.log('Now you can run ' + config.decorators.gray('blinkloader server') + ' to check optimization results.');
	config.config.removeDir(config.config.blinkloaderDir());
	config.decorators.quit();
};

// Cancel optimization: remove optimized pages, domainRecord, optimized site dir
const cancel = config => err => {
	const done = err => {
		if (err) {
			config.decorators.error(err);
		}
		config.decorators.message(config.decorators.yellow('INFO') + '      ', 'you can open an issue here: ' + config.decorators.gray('https://github.com/blinkloader/blinkloader/issues'));
		config.decorators.quit();
	};
	deleteOptimization(config)
		.then(removeDir(config.config.blinkloaderDir(), config))
		.then(removeDir(config.config.getSiteOptimizedDir(), config))
		.then(removeGlob(config.config.getSiteArchive() + '*'))
		.then(() => done(err))
		.catch(err => done(err));
};

const optimize = () => {
	// Gracefull teardown
	kill(signal => {
		cancel(config)('aborted: ' + signal);
	});

	// Graceful teardown on windows
	if (process.platform === 'win32') {
		const rint = require('readline').createInterface(process.stdin, {});
		rint.input.on('keypress', char => {
			if (char === '\u0003') {
				cancel(config)('aborted: SIGINT');
			}
		});
		require('tty').setRawMode(true);
	}

	decorators.n();

	if (config.config.get() === null) {
		decorators.errorExit('no blinkloader.json file in project directory, generate it with ' + decorators.gray('blinkloader init') + ' command');
	}

	if (!fs.existsSync(config.config.blinkloaderDir())) {
		fs.mkdirSync(config.config.blinkloaderDir());
	}

	// Figuring out static server directory
	decorators.log('static website directory: ' + decorators.gray(config.config.getSiteDir()));
	decorators.log('output directory: ' + decorators.gray(config.config.getSiteOptimizedDir()));

	// Creating '{site directory name}_optimized' directory
	config.config.copyDir(config.config.getSiteDir(), config.config.getSiteOptimizedDir());

	// Load all html assets names into memory
	config.pages = config.config.getDirEntries(config.config.getSiteDir(), config.config.walkDir(config.config.getSiteDir()), ['html', 'htm']);

	// Load all js, css assets names into memory
	config.assets = config.config.getDirEntries(config.config.getSiteDir(), config.config.walkDir(config.config.getSiteDir()), ['js', 'css']);

	const totalTargets = config.config.getDirEntries(config.config.getSiteDir(), config.config.walkDir(config.config.getSiteDir()), ['html', 'htm', 'png', 'jpg', 'jpeg', 'js', 'css']);
	decorators.log('identified ' + decorators.gray(totalTargets.length) + ' optimization target' + (totalTargets.length > 1 ? 's' : ''));
	decorators.n();

	checkAPIQuota(config)
		.then(createOptimization)
		.then(prepareOptimization)
		.then(startOptimization)
		.then(waitOptimization)
		.then(fetchPageDownloadLinks)
		.then(downloadPages)
		.then(minifyAssets)
		.then(gzipAssets)
		.then(deleteOptimization)
		.then(removeGlob(config.config.getSiteArchive() + '*'))
		.then(optimizationSuccess)
		.catch(cancel(config));
};

module.exports = optimize;
