import exit from 'exit';
import Config from '../../utils/config';
import {n, gray, error, errorExit, message, yellow, checkbox, green, red} from '../../utils/decorators';

const url = require('url');
const fs = require('fs');

const express = require('express');
const freeport = require('freeport');
const compression = require('compression');
const ncp = require('copy-paste');
const responseTime = require('response-time');
const pretty = require('prettysize');

const exts = ['.js', '.css', '.html', '.htm', '.png', '.jpg', '.jpeg', '.gif'];

const isPage = name => {
	if (name.indexOf('.') === -1) {
		return true;
	}
	return false;
};

const hasExt = (name, ext) => {
	const di = name.lastIndexOf('.');
	const qi = name.lastIndexOf('?');
	if (di !== -1) {
		if (qi !== -1) {
			name = name.substr(0, qi);
		}
		if (name.endsWith(ext)) {
			return true;
		}
	}
	return false;
};

const toGzip = name => {
	for (let i = 0; i < exts.length; i++) {
		if (hasExt(name, exts[i]) || isPage(name)) {
			return true;
		}
	}
	return false;
};
const app = express();

module.exports = args => {
	const config = new Config('./blinkloader.json');
	if (config.get() === null) {
		errorExit('no blinkloader.json file in project directory, generate it with ' + gray('blinkloader init') + ' command');
	}

	const optimizedDir = config.getSiteOptimizedDir();

	if (!fs.existsSync(optimizedDir)) {
		errorExit('optimized website directory could not be found, generate it with ' + gray('blinkloader optimize') + ' command');
	}

	const images = ['png', 'jpg', 'jpeg', 'gif', 'webp'];
	const assets = ['css', 'js'];
	const pages = ['html'];

	if (config.getOptimizations().indexOf('gzip') === -1) {
		app.use(compression());
	}

	app.use(responseTime((req, res, time) => {
		const {method, url} = req;
		const {statusCode} = res;
		const contentLength = res.get('content-length');
		const formattedStatusCode = (statusCode < 300) ? green(statusCode) : yellow(statusCode);
		const formattedSize = (contentLength < 5000000) ? green(pretty(contentLength)) : red(pretty(contentLength));
		message(`${green(method)}       `, `${gray(url)} ${formattedStatusCode} ${parseInt(time, 10)} ms - ${(contentLength && formattedSize) || ''}`);
	}));

	app.use((req, res, next) => {
		const pathname = url.parse(req.url).pathname;
		const extension = pathname.split('.').slice(-1)[0];

		if (images.includes(extension)) {
			res.setHeader('Cache-Control', 'public, max-age=86400');
		} else if (assets.includes(extension)) {
			res.setHeader('Cache-Control', 'public, max-age=31536000');
		} else if (pages.includes(extension)) {
			res.setHeader('Cache-Control', 'public, no-cache');
		} else {
			res.setHeader('Cache-Control', 'public, no-cache');
		}

		if (config.getOptimizations().indexOf('gzip') !== -1) {
			if (toGzip(req.originalUrl)) {
				res.setHeader('Content-Encoding', 'gzip');
			}
		}

		next();
	});

	app.use((req, res, next) => {
		res.setHeader('X-Powered-By', 'Blinkloader');
		next();
	});

	app.use(express.static(optimizedDir));

	freeport((err, port) => {
		port = args.port || port;

		if (err) {
			error(err.message);
			message(yellow('INFO') + '      ', 'you can open an issue here: ' + gray('https://github.com/blinkloader/blinkloader/issues'));
			n();
			exit(0);
		}

		app.listen(port, () => {
			const localhostLink = `http://localhost:${port}`;

			checkbox(`blinkloader started serving ${gray(optimizedDir)} at port ${gray(port)}`);
			ncp.copy(localhostLink, () => {
				checkbox(`copied ${gray(localhostLink)} to clipboard, use Ctrl-V to paste anywhere`);
				n();
			});
		});
	});

	n();
};
